#include "StdAfx.h"
#include "ShortestPath.h"

CShortestPath::CShortestPath()
{
}


CShortestPath::~CShortestPath()
{
}

void CShortestPath::BuildGraph()
{
	//////////////////////////////////////////////////////////////////////////
	// Vertex ߰ϱ
	const VertexDescriptor vA  = boost::add_vertex( m_Graph );
	const VertexDescriptor vB  = boost::add_vertex( m_Graph );
	const VertexDescriptor vC  = boost::add_vertex( m_Graph );
	const VertexDescriptor vD  = boost::add_vertex( m_Graph );
	const VertexDescriptor vE  = boost::add_vertex( m_Graph );

	// Vertex ̸ ϱ
	boost::put( boost::vertex_name, m_Graph, vA, "A" );
	boost::put( boost::vertex_name, m_Graph, vB, "B" );
	boost::put( boost::vertex_name, m_Graph, vC, "C" );
	boost::put( boost::vertex_name, m_Graph, vD, "D" );
	boost::put( boost::vertex_name, m_Graph, vE, "E" );



	//////////////////////////////////////////////////////////////////////////
	// Edge ߰ϱ
	const EdgeDescriptor eAC = boost::add_edge( vA, vC, m_Graph ).first;
	const EdgeDescriptor eBB = boost::add_edge( vB, vB, m_Graph ).first;
	const EdgeDescriptor eBD = boost::add_edge( vB, vD, m_Graph ).first;
	const EdgeDescriptor eBE = boost::add_edge( vB, vE, m_Graph ).first;
	const EdgeDescriptor eCB = boost::add_edge( vC, vB, m_Graph ).first;
	const EdgeDescriptor eCD = boost::add_edge( vC, vD, m_Graph ).first;
	const EdgeDescriptor eDE = boost::add_edge( vD, vE, m_Graph ).first;
	const EdgeDescriptor eEB = boost::add_edge( vE, vB, m_Graph ).first;
	const EdgeDescriptor eEA = boost::add_edge( vE, vA, m_Graph ).first;

	// Edge ̸ ϱ
	boost::put( boost::edge_name, m_Graph, eAC, "AC" );
	boost::put( boost::edge_name, m_Graph, eBB, "BB" );
	boost::put( boost::edge_name, m_Graph, eBD, "BD" );
	boost::put( boost::edge_name, m_Graph, eBE, "BE" );
	boost::put( boost::edge_name, m_Graph, eCB, "CB" );
	boost::put( boost::edge_name, m_Graph, eCD, "CD" );
	boost::put( boost::edge_name, m_Graph, eDE, "DE" );
	boost::put( boost::edge_name, m_Graph, eEB, "EB" );
	boost::put( boost::edge_name, m_Graph, eEA, "EA" );
	
	// Edge ġ ϱ
	boost::put( boost::edge_weight, m_Graph, eAC, 1 );
	boost::put( boost::edge_weight, m_Graph, eBB, 2 );
	boost::put( boost::edge_weight, m_Graph, eBD, 1 );
	boost::put( boost::edge_weight, m_Graph, eBE, 2 );
	boost::put( boost::edge_weight, m_Graph, eCB, 7 );
	boost::put( boost::edge_weight, m_Graph, eCD, 3 );
	boost::put( boost::edge_weight, m_Graph, eDE, 1 );
	boost::put( boost::edge_weight, m_Graph, eEB, 1 );
	boost::put( boost::edge_weight, m_Graph, eEA, 1 );


	//////////////////////////////////////////////////////////////////////////
	// Shortest Path ϱ

	// Property Map 
	boost::property_map< Graph, boost::edge_weight_t >::type		mapEdgeWeight			= boost::get( boost::edge_weight, m_Graph );
	boost::property_map< Graph, boost::vertex_index_t >::type		mapVertexIndex			= boost::get( boost::vertex_index, m_Graph );
	boost::property_map< Graph, boost::vertex_distance_t >::type	mapVertexDistance		= boost::get( boost::vertex_distance, m_Graph );
	boost::property_map< Graph, boost::vertex_predecessor_t >::type	mapVertexPredecessor	= boost::get( boost::vertex_predecessor, m_Graph );

	// ͽƮ ִܰŸ ˰
	const VertexDescriptor vStartVertex = vA;
	boost::dijkstra_shortest_paths(
		m_Graph, 
		vStartVertex,
		boost::weight_map( mapEdgeWeight ).vertex_index_map( mapVertexIndex ).distance_map( mapVertexDistance ).predecessor_map( mapVertexPredecessor )
		); 


	//////////////////////////////////////////////////////////////////////////
	// ִܰŸ  
	PrintShortestPath( vStartVertex );

	// Dot Ϸ 
	WriteGraphviz();
}


void CShortestPath::ResolvePath( const VertexDescriptor& vStart, const VertexDescriptor& vEnd )
{
	std::cout << "ResolvePath : ";
	std::vector< VertexDescriptor > vecResolvePath;

	// vEnd -> vStart  θ ŽѴ.
	for( VertexDescriptor v_it = vEnd; v_it != vStart; v_it = boost::get( boost::vertex_predecessor, m_Graph, v_it ) )
	{
		vecResolvePath.push_back( v_it );
	}
	vecResolvePath.push_back( vStart );

	// θ  Ѵ.
	for( std::vector< VertexDescriptor >::const_reverse_iterator it = vecResolvePath.rbegin();
		 it != vecResolvePath.rend();
		 ++it )
	{
		std::string strVertexName( boost::get( boost::vertex_name, m_Graph, (*it) ) );
		std::cout << strVertexName.c_str() << " ";
	}

	std::cout << std::endl;
}

void CShortestPath::PrintShortestPath( const VertexDescriptor& vStartVertex )
{
	Graph::vertex_iterator v_it, v_it_end;
	for( boost::tie( v_it, v_it_end ) = boost::vertices( m_Graph ); v_it != v_it_end; ++v_it )
	{
		std::string strStartVertexName( boost::get( boost::vertex_name, m_Graph, vStartVertex ) );
		std::string strEndVertexName( boost::get( boost::vertex_name, m_Graph, (*v_it) ) );

		std::size_t sizeDistance = boost::get( boost::vertex_distance, m_Graph, (*v_it) );

		std::cout << "Distance( Start : " << strStartVertexName.c_str() << " End : " << strEndVertexName.c_str() << " ) = " << sizeDistance << std::endl;

		// ִܰŸ 
		ResolvePath( vStartVertex, (*v_it) );
	}
}

void CShortestPath::WriteGraphviz()
{
 	std::ofstream ofstream( "Graphviz.dot", std::ios::out | std::ios::trunc );
	if( !ofstream )
	{
		std::cout << "Could not open file." << std::endl;
		assert( 0 );
		return;
	}

 	boost::write_graphviz( ofstream, m_Graph, vertexWriter( m_Graph ), edgeWriter( m_Graph ), graphWriter() );

	ofstream.close();
}
