#pragma once

namespace
{	
	typedef int		Weight;

	// Type ( Graph, Vertex, Edge )
	typedef boost::adjacency_list <
		boost::vecS, 
		boost::vecS, 
		boost::bidirectionalS,
		boost::property< boost::vertex_name_t, std::string,						// Vertex Property 
						 boost::property< boost::vertex_index_t, size_t,
						 boost::property< boost::vertex_distance_t, size_t,
						 boost::property< boost::vertex_predecessor_t, size_t
						 > > > >,
		boost::property< boost::edge_name_t, std::string,						// Edge Property 
						 boost::property< boost::edge_weight_t, Weight > >
	>	Graph; 

	typedef boost::graph_traits< Graph >::vertex_descriptor		VertexDescriptor; 
	typedef boost::graph_traits< Graph >::edge_descriptor		EdgeDescriptor; 


	// vertex & Edge Writer
	struct vertexWriter
	{
		vertexWriter( const Graph& graph ) 
			: _Graph( graph ) {};

		template <typename Vertex>
		void operator()(std::ostream& outStream, Vertex v)
		{
			std::string vertexName( boost::get( boost::vertex_name, _Graph, v ) );
			outStream << "[label=\"" << vertexName.c_str() << "\"]";
		};

	private:
		const Graph& _Graph;
	};

	struct edgeWriter
	{
		edgeWriter( const Graph& graph ) 
			: _Graph( graph ) {};

		template <typename Edge>
		void operator()(std::ostream& outStream, Edge e)
		{
			std::string edgeName( boost::get( boost::edge_name, _Graph, e ) );
			Weight EdgeWeight = boost::get( boost::edge_weight, _Graph, e );
			outStream << "[label=\"" << edgeName.c_str() << " weight = " << EdgeWeight << "\"]";
		}

	private:
		const Graph& _Graph;
	};

	struct graphWriter
	{
		void operator()(std::ostream& outStream) const
		{
			outStream << "graph [bgcolor=white]" << std::endl;
			outStream << "node [shape=record color=blue]" << std::endl;
			// just an example, showing that local options override global
			outStream << "edge [color=red]" << std::endl;
			outStream << "rankdir=LR" << std::endl;
		}
	};
};

class CShortestPath
{
public:
	CShortestPath();
	~CShortestPath();

public:
	void BuildGraph();
	void ResolvePath( const VertexDescriptor& vStart, const VertexDescriptor& vEnd );
	void PrintShortestPath( const VertexDescriptor& vStartVertex );

	void WriteGraphviz();	// Dot Ϸ ϱ

private:
	Graph m_Graph;
};

